@@ -1,22 +1,22 @@
 /**************************************************************************
- *   help.c                                                               *
+ *   help.c  --  This file is part of GNU nano.                           *
  *                                                                        *
  *   Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,  *
  *   2009, 2010, 2011, 2013, 2014 Free Software Foundation, Inc.          *
- *   This program is free software; you can redistribute it and/or modify *
- *   it under the terms of the GNU General Public License as published by *
- *   the Free Software Foundation; either version 3, or (at your option)  *
- *   any later version.                                                   *
+ *   Copyright (C) 2014, 2015, 2016 Benno Schulenberg                     *
  *                                                                        *
- *   This program is distributed in the hope that it will be useful, but  *
- *   WITHOUT ANY WARRANTY; without even the implied warranty of           *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
- *   General Public License for more details.                             *
+ *   GNU nano is free software: you can redistribute it and/or modify     *
+ *   it under the terms of the GNU General Public License as published    *
+ *   by the Free Software Foundation, either version 3 of the License,    *
+ *   or (at your option) any later version.                               *
+ *                                                                        *
+ *   GNU nano is distributed in the hope that it will be useful,          *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty          *
+ *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.              *
+ *   See the GNU General Public License for more details.                 *
  *                                                                        *
  *   You should have received a copy of the GNU General Public License    *
- *   along with this program; if not, write to the Free Software          *
- *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA            *
- *   02110-1301, USA.                                                     *
+ *   along with this program.  If not, see http://www.gnu.org/licenses/.  *
  *                                                                        *
  **************************************************************************/
 
@@ -419,7 +419,11 @@ void help_init(void)
 		 * but allow it to spill into the second, for "M-Space". */
 		if (scsfound == 1) {
 		    sprintf(ptr, "%s              ", s->keystr);
-		    ptr += 6;
+		    /* Unicode arrows take three bytes instead of one. */
+		    if (s->keystr[1] == '\xE2')
+			ptr += 8;
+		    else
+			ptr += 6;
 		} else {
 		    ptr += sprintf(ptr, "(%s)\t", s->keystr);
 		    break;
@@ -482,6 +486,10 @@ functionptrtype parse_help_input(int *kbinput)
 		return do_page_up;
 	    case 'E':
 	    case 'e':
+	    case 'Q':
+	    case 'q':
+	    case 'X':
+	    case 'x':
 		return do_exit;
 	}
     }
